import importlib
import json
import pkgutil
from commonfate_provider import access, resources, target
from commonfate_provider.loader import load_provider
import click


def import_submodules(package, rel_name=None, recursive=True):
    """Import all submodules of a module, recursively, including subpackages

    :param package: package (name or actual module)
    :type package: str | module
    :rtype: dict[str, types.ModuleType]
    """
    if isinstance(package, str):
        package = importlib.import_module(package, rel_name)
    results = {}
    for loader, name, is_pkg in pkgutil.walk_packages(package.__path__):
        full_name = package.__name__ + "." + name
        results[full_name] = importlib.import_module(full_name)
        if recursive and is_pkg:
            results.update(import_submodules(full_name))
    return results


@click.command()
@click.option("--dir", default=".", help="Directory to the load the provider from")
def schema(dir):
    Provider = load_provider(dir)
    schema = {}

    # in future we'll handle multiple kinds of targets,
    # but for now, just get the first one
    target_kind = next(iter(access._ALL_TARGETS))
    target_class = access._ALL_TARGETS[target_kind]

    schema["target"] = target.export_schema(target_kind, target_class)

    schema["config"] = Provider.export_schema()
    schema["audit"] = resources.audit_schema()
    print(json.dumps(schema))


@click.group()
def cli():
    pass


cli.add_command(schema)

if __name__ == "__main__":
    cli()



from commonfate_provider.runtime import AWSLambdaRuntime
from commonfate_provider import provider, access, target


class BasicProvider(provider.Provider):
    pass


@access.target(kind="Default")
class Args:
    group = target.String()
    pass


@access.grant()
def grant(p: BasicProvider, subject, args):
    pass


basic_provider = BasicProvider()

runtime = AWSLambdaRuntime(provider=basic_provider, config_loader=provider.NoopLoader())


def test_lambda_handler_works():
    event = {
        "type": "grant",
        "data": {
            "subject": "testuser",
            "target": {"arguments": {"group": "test"}, "kind": "Default"},
        },
    }
    runtime.handle(event=event, context=None)


def test_provider_describe():
    event = {"type": "describe"}
    runtime.handle(event=event, context=None)


def test_lambda_runtime_calls_provider_setup():
    class MyProvider(provider.Provider):
        def setup(self):
            self.is_setup = True

    runtime = AWSLambdaRuntime(
        provider=MyProvider(), config_loader=provider.NoopLoader()
    )
    assert runtime.provider.is_setup == True



import os
from commonfate_provider import loader
from commonfate_provider.provider import DictLoader
from commonfate_provider.runtime import AWSLambdaRuntime
import importlib.resources
import json
import importlib
import pkgutil


def import_submodules(package, recursive=True):
    """Import all submodules of a module, recursively, including subpackages

    :param package: package (name or actual module)
    :type package: str | module
    :rtype: dict[str, types.ModuleType]
    """
    if isinstance(package, str):
        package = importlib.import_module(package)
    results = {}
    for loader, name, is_pkg in pkgutil.walk_packages(package.__path__):
        full_name = package.__name__ + "." + name
        results[full_name] = importlib.import_module(full_name)
        if recursive and is_pkg:
            results.update(import_submodules(full_name))
    return results


# this package is generated by the PDK packaging process,
# and will exist in the AWS Lambda deployment zip file.
try:
    import commonfate_provider_dist

    import_submodules(commonfate_provider_dist)
except ImportError as e:
    raise ImportError(
        f"{e}\nUsually this means that the Provider has been incorrectly packaged. Please report this issue to the provider developer."
    )


def to_camel_case(snake_str):
    """
    Split each word by seperator "_" and capitalize the
    first letter of each component with '.title()' method and join the result.

    For example, `snake_case` will be converted to `SnakeCase`
    """
    components = snake_str.split("_")

    return "".join(x.title() for x in components)


def load_metadata_value(provider_data: dict, key: str):
    try:
        val = provider_data[key]
        return val
    except KeyError:
        raise KeyError(
            f"{key} was not found in the manifest.json file. Usually this means that the provider has been incorrectly packaged. Please report this issue to the provider developer."
        )


(Provider, Args) = loader.load_provider_from_subclass()

config_dict = {}
for key in Provider.export_schema():
    config_dict[key] = os.getenv(to_camel_case(key))

config_loader = DictLoader(config_dict=config_dict)
provider = Provider()

with importlib.resources.open_text("commonfate_provider_dist", "manifest.json") as file:
    provider_data = json.load(file)


runtime = AWSLambdaRuntime(
    provider=provider,
    config_loader=config_loader,
    name=load_metadata_value(provider_data, "name"),
    version=load_metadata_value(provider_data, "version"),
    publisher=load_metadata_value(provider_data, "publisher"),
)


def lambda_handler(event, context):
    return runtime.handle(event, context)



import os
import typing
from commonfate_provider.provider import Provider
from importlib.util import spec_from_file_location, module_from_spec
import toml


def load_provider(
    cwd: str = "",
) -> typing.Type[Provider]:
    """
    Loads the Common Fate Provider by reading the provider.toml file.
    This method dynamically instantiates the provider class.
    """
    l = os.path.join(cwd, "provider.toml")
    config = toml.load(l)

    language = config["language"]

    if language != "python3.9":
        # this wrapping code is written in Python, so we can only handle
        # Providers written in Python.
        raise Exception(f"invalid language: {language}")

    provider_class = load_class(cwd, config["python"]["class"])
    return provider_class


def load_class(cwd: str, path: str):
    """
    Dynamically loads a class. The path argument is the path
    to the class in the format 'module.ClassName'
    """
    components = path.split(".")  # [module, ClassName]
    modulePathWithinProviderFolder = "/".join(components[:-1]) + ".py"

    # Import the module using an absolute path to the python file containing the Provider class
    spec = spec_from_file_location(
        "provider", os.path.join(cwd, modulePathWithinProviderFolder)
    )
    # Loading the module

    if spec is None or spec.loader is None:
        raise Exception("expected spec and spec.loader not to be None")
    module = module_from_spec(spec)
    spec.loader.exec_module(module)

    class_name = components[-1]  # ClassName
    my_class = getattr(module, class_name)
    return my_class


def load_provider_from_subclass() -> typing.Type[Provider]:
    """
    Loads the Common Fate Provider.
    This method dynamically instantiates the provider class.
    """
    classes = Provider.__subclasses__()

    if len(classes) == 0:
        raise Exception(
            f"could not find a Provider class. Usually this means that the Provider has been incorrectly packaged. Please report this issue to the provider developer."
        )

    if len(classes) > 1:
        raise Exception(
            f"only 1 Provider class is supported but found {len(classes)}: {[cl.__name__ for cl in classes]}"
        )

    ProviderClass = classes[0]

    return ProviderClass



from commonfate_provider import access


def test_target():
    # reset the registered targets
    access._ALL_TARGETS = {}

    @access.target()
    class ExampleTarget:
        pass

    assert access._ALL_TARGETS == {"ExampleTarget": ExampleTarget}


def test_target_with_kind():
    # reset the registered targets
    access._ALL_TARGETS = {}

    @access.target(kind="SomethingElse")
    class ExampleTarget:
        pass

    assert access._ALL_TARGETS == {"SomethingElse": ExampleTarget}



from dataclasses import dataclass
import typing_extensions
import typing

_ALL_TARGETS: typing.Dict[str, typing.Any] = {}


_T = typing.TypeVar("_T")


@typing_extensions.dataclass_transform()
def target(
    kind: typing.Optional[str] = None,
) -> typing.Callable[[type[_T]], type[_T]]:
    def actual_decorator(cls: type[_T]) -> type[_T]:
        nonlocal kind
        if kind is None:
            kind = cls.__name__
        _ALL_TARGETS[kind] = cls

        return cls

    return actual_decorator


@dataclass
class GrantResult:
    access_instructions: typing.Optional[str] = None
    """
    Instructions on how to access the entitlements.
    Common Fate will display these to the user upon a successful Access Request.
    """


GrantValidatorFunc = typing.Callable[[typing.Any, str, typing.Any], None]


@dataclass
class GrantValidator:
    name: str
    id: str
    func: GrantValidatorFunc


_ALL_GRANT_VALIDATORS: typing.Dict[str, typing.Dict[str, GrantValidator]] = {}


def grant_validator(
    name: str,
    _internal_key: str = "default",
) -> typing.Callable[[GrantValidatorFunc], GrantValidatorFunc]:
    def actual_decorator(func: GrantValidatorFunc):
        id = func.__name__
        cv = GrantValidator(id=id, name=name, func=func)
        _ALL_GRANT_VALIDATORS.setdefault(_internal_key, {})
        _ALL_GRANT_VALIDATORS[_internal_key][id] = cv
        return func

    return actual_decorator


GrantFunc = typing.Callable[[typing.Any, str, typing.Any], typing.Optional[GrantResult]]
_GRANT: typing.Dict[str, GrantFunc] = {}


def _get_grant_func(_internal_key: str = "default"):
    return _GRANT[_internal_key]


def grant(
    _internal_key: str = "default",
) -> typing.Callable[[GrantFunc], GrantFunc]:
    # _internal_key is used for testing purposes, and allows
    # multiple independent decorators to be registered in a single test file
    def actual_decorator(func: GrantFunc):
        _GRANT[_internal_key] = func
        return func

    return actual_decorator


RevokeFunc = typing.Callable[[typing.Any, str, typing.Any], None]


_REVOKE: typing.Dict[str, RevokeFunc] = {}


def revoke(
    _internal_key: str = "default",
) -> typing.Callable[[RevokeFunc], RevokeFunc]:
    # _internal_key is used for testing purposes, and allows
    # multiple independent decorators to be registered in a single test file
    def actual_decorator(func: RevokeFunc):
        _REVOKE[_internal_key] = func
        return func

    return actual_decorator


def _get_revoke_func(_internal_key: str = "default"):
    return _REVOKE[_internal_key]
